package Jellybean::Engine;

use Carp 'croak';
use Class::ActsLike;

use Jellybean::Response;

use vars '$VERSION';
$VERSION = '0.20';

sub new
{
	bless {}, $_[0];
}

sub add_server
{
	my ($self, $server) = @_;
	croak "No server provided" unless $server;
	croak "Invalid server provided"
		unless UNIVERSAL::acts_like( $server, 'Jellybean::Server' );
	push @{ $self->{servers} }, $server;
}

sub servers
{
	my $self = shift;
	return $self->{servers};
}

sub listen
{
	my $self = shift;

	my @requests;
	foreach my $server (@{ $self->servers() })
	{
		next unless   $server->incoming();
		my $request = $server->request();
		push @requests, [ $request, $server ];
	}

	return @requests;
}

sub add_action
{
	my ($self, $name, $action) = @_;
	$self->{actions}{$name}    = $action;
}

sub process_request
{
	my ($self, $request, $server) = @_;

	my $response = Jellybean::Response->new();
	$response->connection( $request->connection() );
	my $path     = $request->path_forward() || '';

	my $action;
	if ( exists $self->{actions}{$path} )
	{
		$action = $self->{actions}{$path};
	}	
	else
	{
		$action = $self->{actions}{default};
		$request->path_back();
	}
	$action->activate( $request, $response );
	$server->respond( $response );
}

1;
