package Jellybean::Server::HTTP;

use strict;
use Carp 'croak';
use Class::ActsLike 'Jellybean::Server';

use IO::Select;
use HTTP::Daemon;
use Jellybean::Request;

sub new
{
	my ($class, %args) = @_;
	croak 'No port specified' unless $args{LocalPort};

	my $daemon = HTTP::Daemon->new( Listen => 10, Reuse => 1, %args )
		or croak "Could not create daemon: $!";

	my $select = IO::Select->new();
	$select->add( $daemon );

	bless { Daemon => $daemon, Select => $select }, $class;
}

sub incoming
{
	my $self = shift;
	return $self->{Select}->can_read() ? 1 : 0;
}

sub request
{
	my $self = shift;
	my $conn = $self->{Daemon}->accept();

	my $uri  = $conn->get_request( 1 )->uri();
	my @path = split('/', $uri->path() ); 
	shift @path;

	return Jellybean::Request->new( path => \@path, conn => $conn );
}

my %http_headers = (
	length => 'Content-length',
	type   => 'Content-type',
);

sub respond
{
	my ($self, $response) = @_;
	croak "No response given" unless $response;
	croak "Invalid response"
		unless UNIVERSAL::acts_like( $response, 'Jellybean::Response' );

	my %headers = $response->headers();
	my $headers = "HTTP/1.1 200 OK\r\n";
	while (my ($name, $value) = each %headers)
	{
		$name = exists $http_headers{$name} ? $http_headers{$name} : $name;
		$headers .= "$name: $value\r\n";
	}
	my $conn    = $response->connection();
	$conn->print( $headers . "\r\n\r\n", $response->body() );
}

1;
