#!/usr/bin/perl -w

BEGIN
{
	chdir 't' if -d 't';
	use lib '../lib', '../blib/lib';
}

use strict;

use File::Path;
use File::Spec;

use Test::More tests => 15;
use Test::MockObject;

my $mock_in  = Test::MockObject->new();
my $mock_out = Test::MockObject->new();

my $module = 'Jellybean::Container::File';
use_ok( $module ) or exit;

can_ok( $module, 'new' );
my $f = $module->new();
isa_ok( $f, $module );

can_ok( $f, 'activate' );

my @path = qw( files onedir twodir );
mkpath [ File::Spec->catdir( @path ) ];
push @path, 'file.html';

write_file( \@path, <<END_HERE );
<html>
<head><title>Demo HTML</title></head>
<body><p>Please enjoy my HTML page.</p></body>
</html>
END_HERE

$mock_in->mock( path => sub { @path } );
$mock_out->set_true( 'add_header' )
	     ->set_true( 'body' );

$f->activate( $mock_in, $mock_out );

my ($method, $args) = $mock_in->next_call();
is( $method, 'path',         'activate() should get path' );

($method, $args)    = $mock_out->next_call();
is( $method, 'add_header',   '... adding header'   );
is( $args->[1], 'length',    '... for file length' );
is( $args->[2], 100,         '... accurately'      );

($method, $args)    = $mock_out->next_call();
is( $method, 'add_header',   '... adding header'   );
is( $args->[1], 'type',      '... for file type'   );
is( $args->[2], 'text/html', '... accurately'      );

($method, $args)    = $mock_out->next_call();
is( $method, 'body',         '... setting body'    );
like( $args->[1], qr/Plea/,  '... in one swoop'    );

# now start by giving it a directory
$f    = $module->new( basedir => 'files' );
$path[-1] = 'foo.txt';
write_file( \@path, "Here is my\nsuper happy\ntext file.\n" );
shift @path;

$f->activate( $mock_in, $mock_out );
($method, $args)   = $mock_out->next_call( 2 );
is( $args->[2], 'text/plain',     '... setting text file type too' );

($method, $args)    = $mock_out->next_call();
like( $args->[1], qr/Here is my/, '... using base dir when provided' );

sub write_file
{
	my ($path, $contents) = @_;
	local *OUT;
	open( OUT, '> ' . File::Spec->catfile( @$path ) );
	print OUT $contents;
}

END
{
	rmtree 'files';
}
