#!/usr/bin/perl -w

BEGIN
{
	chdir 't' if -d 't';
	use lib '../lib', '../blib/lib';
}

use strict;

use Test::More tests => 16;
use Test::Exception;
use Test::MockObject;

my $mock = Test::MockObject->new();

my $module = 'Jellybean::Request';
use_ok( $module ) or exit;
can_ok( $module, 'new' );

throws_ok { $module->new() } qr/No connection/,
	'new() should croak without connection passed';

my $r = $module->new( conn => $mock );
isa_ok( $r, $module );
can_ok( $r, 'path' );
my @result = $r->path();
is( @result, 0, 'default path should be empty' );

$r = $module->new( path => [qw( path to foo )], conn => $mock );
@result = $r->path();
is_deeply( \@result, [qw( path to foo )],
	'path() should return path set in constructor' );

can_ok( $r, 'connection' );
is( $r->connection(), $mock,
	'connection() should return connection from constructor' );

can_ok( $r, 'path_forward' );
my $response = $r->path_forward();
is_deeply( [ $r->path() ], [qw( to foo )],
	'path_forward() should move ahead one step in path' );
is( $response, 'path', '... returning the step it took' );
$r->path_forward();
$r->path_forward();
$r->path_forward();
is_deeply( [ $r->path() ], [],
	'... but not if at end already' );

can_ok( $r, 'path_back' );
$r = $module->new( path => [qw( path to foo )], conn => $mock );
$r->path_forward();
$r->path_back();

is_deeply( [ $r->path() ], [qw( path to foo )],
	'path_back() should move back one step in path' );
$r->path_back();
is_deeply( [ $r->path() ], [qw( path to foo )],
	'... but not if at start already' );

__END__
