#!/usr/bin/perl -w

BEGIN
{
	chdir 't' if -d 't';
	use lib '../lib', '../blib/lib';
}

use strict;

use Test::More tests => 26;

use URI;
use Class::ActsLike;

use Test::Exception;
use Test::MockObject;

my $mock = Test::MockObject->new();
$mock->fake_module( 'HTTP::Daemon', new => sub {shift; $mock->newdaemon( @_ )});
$mock->set_series( newdaemon => $mock, 0 );

my $mock_select = Test::MockObject->new();
$mock_select->fake_module( 'IO::Select', new => sub { $mock_select } );
local *Test::MockObject::add;
$mock_select->set_true( 'add' );

my $mock_req = Test::MockObject->new();
$mock_req->fake_module( 'Jellybean::Request',
	new => sub { shift; $mock_req->jrnew( @_ ) } );

my $module = 'Jellybean::Server::HTTP';
use_ok( $module ) or exit;
can_ok( $module,  'new' );

throws_ok { $module->new() } qr/No port specified/,
	                      'new() should die if no port is given';

my $s = $module->new( LocalPort => 8000 );
isa_ok( $s, $module,      '... otherwise creating an object that' ) ;

my ($method, $args) = $mock->next_call();
is( $method, 'newdaemon', '... creating a new HTTP::Daemon' );
shift @$args;
my %args = @$args;
is( $args{LocalPort}, 8000,    '... passing along the specified port' );
is( $args{Reuse},        1,    '... and the default Reuse value' );
is( $args{Listen},      10,    '... and the default Listen value' );

throws_ok { $module->new( LocalPort => 9000 ) } qr/Could not create daemon/,
	                      '... croaking unless daemon can be created';

($method, $args) = $mock_select->next_call();
is( $method, 'add',       '... adding to IO::Select' );
is( $args->[1], $mock,    '... the daemon' );

can_ok( $module, 'incoming' );

$mock_select->set_series( can_read => $mock_select, 0 );

ok(   $s->incoming(), 'incoming() should return true with a pending request' );
ok( ! $s->incoming(), '... false otherwise' );

can_ok( $module, 'request' );
my $mock_conn = Test::MockObject->new();
$mock->set_always( accept => $mock_conn );
$mock_conn->set_always( get_request => $mock_conn )
	      ->set_always( uri => URI->new( 'http://example.com/some/path' ) )
	      ->set_true( 'close' );

$mock_req->set_always( jrnew => $mock_req );

my $result = $s->request();
is( $result, $mock_req,
	'request() should return a new Jellybean::Request' );
($method, $args) = $mock_req->next_call();
shift @$args;
%args = @$args;
is_deeply( $args{path}, [qw( some path )], '... setting the path' );
is( $args{conn}, $mock_conn,   '... and storing the connection' );

# method
# content_type
# content_length
# query
# cookie
# cookies

can_ok( $module, 'respond' );
my $mock_resp = Test::MockObject->new();
{
	package Test::MockObject;
	Class::ActsLike->import( 'Jellybean::Response' );
}

throws_ok { $s->respond() } qr/No response given/,
	'respond() should croak given no response';

throws_ok { $s->respond( 'foo' ) } qr/Invalid response/,
	'... or with invalid response';

$mock_resp->mock( headers    => sub {
	( 'content', 'foo', 'length', 50, 'type', 'fake' )} )
	      ->set_always( body       => 'lots of text' )
		  ->set_always( connection => $mock_conn );

$mock_conn->set_true( 'print' )
	      ->clear();

$s->respond( $mock_resp );
($method, $args) = $mock_conn->next_call();
is( $method, 'print', '... printing response to connection' );
like( $args->[1], qr/content: foo/, '... formatting given headers ' );
like( $args->[1], qr/length: 50/,   '... formatting given headers ' );
like( $args->[1], qr/type: fake/,   '... formatting given headers ' );
is(   $args->[2], 'lots of text',   '... and printing content' );
